<?php

namespace Assetdocs\Company\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Company\Http\Requests\CompanyAdminWebRequest;
use Assetdocs\Company\Interfaces\CompanyRepositoryInterface;
use Assetdocs\Company\Models\Company;

/**
 * Admin web controller class.
 */
class CompanyAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize company controller.
     *
     * @param type CompanyRepositoryInterface $company
     *
     * @return type
     */
    public function __construct(CompanyRepositoryInterface $company)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $company;
        parent::__construct();
    }

    /**
     * Display a list of company.
     *
     * @return Response
     */
    public function index(CompanyAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $companies  = $this->repository
            ->pushCriteria(new \Assetdocs\Company\Repositories\Criteria\CompanyAdminCriteria())
                ->setPresenter('\\Assetdocs\\Company\\Repositories\\Presenter\\CompanyListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $companies['recordsTotal']    = $companies['meta']['pagination']['total'];
            $companies['recordsFiltered'] = $companies['meta']['pagination']['total'];
            $companies['request']         = $request->all();
            return response()->json($companies, 200);

        }

        $this   ->theme->prependTitle(trans('company::company.names').' :: ');
        return $this->theme->of('company::admin.company.index')->render();
    }

    /**
     * Display company.
     *
     * @param Request $request
     * @param Model   $company
     *
     * @return Response
     */
    public function show(CompanyAdminWebRequest $request, Company $company)
    {
        if (!$company->exists) {
            return response()->view('company::admin.company.new', compact('company'));
        }

        Form::populate($company);
        return response()->view('company::admin.company.show', compact('company'));
    }

    /**
     * Show the form for creating a new company.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CompanyAdminWebRequest $request)
    {

        $company = $this->repository->newInstance([]);

        Form::populate($company);

        return response()->view('company::admin.company.create', compact('company'));

    }

    /**
     * Create new company.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CompanyAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $company          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('company::company.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/company/company/'.$company->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show company for editing.
     *
     * @param Request $request
     * @param Model   $company
     *
     * @return Response
     */
    public function edit(CompanyAdminWebRequest $request, Company $company)
    {
        Form::populate($company);
        return  response()->view('company::admin.company.edit', compact('company'));
    }

    /**
     * Update the company.
     *
     * @param Request $request
     * @param Model   $company
     *
     * @return Response
     */
    public function update(CompanyAdminWebRequest $request, Company $company)
    {
        try {

            $attributes = $request->all();

            $company->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('company::company.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/company/company/'.$company->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/company/company/'.$company->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the company.
     *
     * @param Model   $company
     *
     * @return Response
     */
    public function destroy(CompanyAdminWebRequest $request, Company $company)
    {

        try {

            $t = $company->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('company::company.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/company/company/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/company/company/'.$company->getRouteKey()),
            ], 400);
        }
    }
}
