<?php

namespace Assetdocs\Company\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Company\Http\Requests\CompanyUserWebRequest;
use Assetdocs\Company\Interfaces\CompanyRepositoryInterface;
use Assetdocs\Company\Models\Company;

class CompanyUserController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'web';

    /**
     * Initialize company controller.
     *
     * @param type CompanyRepositoryInterface $company
     *
     * @return type
     */
    public function __construct(CompanyRepositoryInterface $company)
    {
        $this->middleware('web');
        $this->middleware('auth:web');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $company;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(CompanyUserWebRequest $request)
    {
        $this->repository->pushCriteria(new \Assetdocs\Company\Repositories\Criteria\CompanyUserCriteria());
        $companies = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('company::company.names').' :: ');

        return $this->theme->of('company::user.company.index', compact('companies'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Company $company
     *
     * @return Response
     */
    public function show(CompanyUserWebRequest $request, Company $company)
    {
        Form::populate($company);

        return $this->theme->of('company::user.company.show', compact('company'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CompanyUserWebRequest $request)
    {

        $company = $this->repository->newInstance([]);
        Form::populate($company);

        return $this->theme->of('company::user.company.create', compact('company'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CompanyUserWebRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $company = $this->repository->create($attributes);

            return redirect(trans_url('/user/company/company'))
                -> with('message', trans('messages.success.created', ['Module' => trans('company::company.name')]))
                -> with('code', 201);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Company $company
     *
     * @return Response
     */
    public function edit(CompanyUserWebRequest $request, Company $company)
    {

        Form::populate($company);

        return $this->theme->of('company::user.company.edit', compact('company'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Company $company
     *
     * @return Response
     */
    public function update(CompanyUserWebRequest $request, Company $company)
    {
        try {
            $this->repository->update($request->all(), $company->getRouteKey());

            return redirect(trans_url('/user/company/company'))
                ->with('message', trans('messages.success.updated', ['Module' => trans('company::company.name')]))
                ->with('code', 204);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(CompanyUserWebRequest $request, Company $company)
    {
        try {
            $this->repository->delete($company->getRouteKey());
            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('company::company.name')]),
                'code'     => 202,
                'redirect' => trans_url('/user/company/company/0'),
            ], 202);

        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
